@@ -6,7 +6,7 @@ use strict;
 use warnings;
 use English qw< -no_match_vars >;
 use Data::Dumper;
-our $VERSION = '0.728';
+our $VERSION = '0.730';
 
 use Template::Perlish;
 use Log::Log4perl::Tiny qw< :easy :dead_if_first get_logger >;
@@ -106,7 +106,7 @@ sub tubify {
       ($ref eq 'CODE')
         ? $_
         : tube(($ref eq 'ARRAY') ? @$_ : $_)
-   } @_;
+   } grep { $_ } @_;
 } ## end sub tubify
 
 1;
